package com.example.fieldagent.ui.damagedetails

import androidx.lifecycle.ViewModel
import com.example.fieldagent.data.apis.WebService
import com.example.fieldagent.data.models.responses.ItemList
import com.example.fieldagent.data.models.responses.SelectCategoryData
import com.example.fieldagent.data.models.responses.SimpleResponseData
import com.example.fieldagent.data.network.responseUtil.ApiUtils
import com.example.fieldagent.data.network.responseUtil.Resource
import com.example.fieldagent.di.SingleLiveEvent
import dagger.hilt.android.lifecycle.HiltViewModel
import okhttp3.MultipartBody
import okhttp3.RequestBody
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.util.HashMap
import javax.inject.Inject

@HiltViewModel
class DamageDetailsFragmentVM  @Inject constructor(private val webService: WebService) : ViewModel() {

    val addDamage by lazy { SingleLiveEvent<Resource<SimpleResponseData>>() }
    val categoryList by lazy { SingleLiveEvent<Resource<ItemList>>() }
    val selectCategoryList by lazy { SingleLiveEvent<Resource<SelectCategoryData>>() }


    fun addDamage(hashMap: HashMap<String, RequestBody>, imageParts: ArrayList<MultipartBody.Part>) {
        addDamage.value = Resource.loading()

        webService.createDamage(hashMap,imageParts)
            .enqueue(object : Callback<SimpleResponseData> {

                override fun onResponse(call: Call<SimpleResponseData>,
                                        response: Response<SimpleResponseData>
                ) {
                    if (response.isSuccessful) {
                        addDamage.value = Resource.success(response.body())

                    } else {
                        addDamage.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<SimpleResponseData>, throwable: Throwable) {
                    addDamage.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }


    fun editDamage(id: String,hashMap: HashMap<String, RequestBody>, imageParts: ArrayList<MultipartBody.Part>,) {
        addDamage.value = Resource.loading()

        webService.updateDamage(id,hashMap,imageParts)
            .enqueue(object : Callback<SimpleResponseData> {

                override fun onResponse(call: Call<SimpleResponseData>,
                                        response: Response<SimpleResponseData>
                ) {
                    if (response.isSuccessful) {
                        addDamage.value = Resource.success(response.body())

                    } else {
                        addDamage.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<SimpleResponseData>, throwable: Throwable) {
                    addDamage.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }


    fun getCategoryList() {
        categoryList.value = Resource.loading()

        webService.getCategoryList()
            .enqueue(object : Callback<ItemList> {

                override fun onResponse(call: Call<ItemList>,
                                        response: Response<ItemList>
                ) {
                    if (response.isSuccessful) {
                        categoryList.value = Resource.success(response.body())

                    } else {
                        categoryList.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<ItemList>, throwable: Throwable) {
                    categoryList.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }

    fun getSelectCategoryList(hashMap: HashMap<String, String>) {
        selectCategoryList.value = Resource.loading()

        webService.getSelectCategoryList(hashMap)
            .enqueue(object : Callback<SelectCategoryData> {

                override fun onResponse(call: Call<SelectCategoryData>,
                                        response: Response<SelectCategoryData>
                ) {
                    if (response.isSuccessful) {
                        selectCategoryList.value = Resource.success(response.body())

                    } else {
                        selectCategoryList.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<SelectCategoryData>, throwable: Throwable) {
                    selectCategoryList.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }





}