package com.example.fieldagent.ui.damagedetails.adapter

import android.net.Uri
import android.util.Log
import android.view.LayoutInflater
import android.view.ViewGroup
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.example.fieldagent.databinding.DamageImageAdapterBinding
import com.example.fieldagent.ui.damagedetails.DamageDetailsFragment
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.visible
import androidx.core.net.toUri
import com.example.fieldagent.utils.IMAGE_BASE_URL
import com.example.fieldagent.utils.showFullScreenImage


class DamageDetailsAdapter(
    val imageList: ArrayList<String?>,
    val damageDetailsFragment: DamageDetailsFragment
) : RecyclerView.Adapter<DamageDetailsAdapter.DamageViewHolder>() {

    class DamageViewHolder(val binding: DamageImageAdapterBinding) : RecyclerView.ViewHolder(binding.root) {

        fun bind(imageUrl: String) {
            binding.rlUploadImage.gone()
            binding.cardImage.visible()
            binding.ivCloseImage.visible()


            val context = binding.root.context

            val uri = if (imageUrl.startsWith("content://") || imageUrl.startsWith("file://")) {
                imageUrl.toUri()
            } else {
                imageUrl
            }

            Glide.with(context)
                .load(uri)
                .into(binding.ivSetImage)

        }
    }

    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): DamageViewHolder {
        val binding = DamageImageAdapterBinding.inflate(LayoutInflater.from(parent.context), parent, false)
        return DamageViewHolder(binding)
    }

    override fun onBindViewHolder(holder: DamageViewHolder, position: Int) {
        if (imageList[position] != null) {

            val imageUrl = imageList[position]

            imageUrl?.let {
                holder.bind(it)
            }
            holder.binding.ivCloseImage.setOnClickListener {
                damageDetailsFragment.removeImage(position)
            }

            holder.binding.cardImage.setOnClickListener {
                showFullScreenImage(imageList[position].toString(),holder.binding.root.context)
            }
        } else {
            holder.binding.rlUploadImage.visible()
            holder.binding.cardImage.gone()
            holder.binding.ivCloseImage.gone()

            holder.binding.rlUploadImage.setOnClickListener {
                damageDetailsFragment.clickItem(position)
            }
        }

    }

    override fun getItemCount(): Int = 5
}