package com.example.fieldagent.ui.damagelist

import android.app.DownloadManager
import android.content.ContentValues
import android.content.Context
import android.content.res.Resources
import android.graphics.Bitmap
import android.graphics.Canvas
import android.graphics.Color
import android.graphics.Paint
import android.graphics.pdf.PdfDocument
import android.net.Uri
import android.os.Bundle
import android.os.Environment
import android.provider.MediaStore
import android.util.Log
import android.view.Gravity
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.Toast
import androidx.fragment.app.viewModels
import androidx.lifecycle.Observer
import androidx.navigation.fragment.findNavController
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.R
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.databinding.FragmentDamageListBinding
import com.example.fieldagent.ui.damagelist.adapter.DamageListAdapter
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.dialogs.ProgressDialog
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.hideShowView
import com.example.fieldagent.utils.showConfirmDialog
import com.example.fieldagent.utils.visible
import com.google.gson.Gson
import dagger.hilt.android.AndroidEntryPoint
import java.io.File
import java.io.FileOutputStream
import java.text.SimpleDateFormat
import java.util.Date
import java.util.HashMap
import java.util.Locale
import javax.inject.Inject
import androidx.core.graphics.toColorInt
import androidx.core.graphics.createBitmap
import androidx.core.text.layoutDirection
import com.example.fieldagent.utils.DOWNLOAD_URL
import okhttp3.OkHttpClient


import java.io.IOException

import okhttp3.ResponseBody
import retrofit2.Retrofit

import retrofit2.*
import retrofit2.converter.gson.GsonConverterFactory

import retrofit2.http.GET
import retrofit2.http.Streaming
import retrofit2.http.Url
import java.io.*
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response


@AndroidEntryPoint
class DamageListFragment : Fragment() {

    private var _binding: FragmentDamageListBinding? = null

    private val binding get() = _binding!!

    private  val viewModel: DamageListFragmentVM by viewModels()

    @Inject
    lateinit var prefsManager: PrefsManager

    private lateinit var progressDialog: ProgressDialog

    private lateinit var adapter: DamageListAdapter

    private var inspectionId=""
    private var clickDeletePos=-1
    private var vatValue=""
    private var hashCode=""

    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentDamageListBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)


        initialize()
        listener()
        bindObservers()

    }

   private fun initialize(){

       progressDialog = ProgressDialog(requireActivity())
       adapter = DamageListAdapter(this)
       binding.rvDamageList.adapter = adapter

       inspectionId = arguments?.getString("id").toString()
       val name = arguments?.getString("name").toString()

       // Use the data
       name.let {
          binding.txtTitle.text=name
       }
       val hashMap = HashMap<String, String>()
       hashMap["id"] = inspectionId
       viewModel.getDamagesList(hashMap)
    }


   private fun bindObservers(){
       viewModel.damages.observe(requireActivity(), Observer {
           it ?: return@Observer
           when (it.status) {
               Status.SUCCESS -> {

                   progressDialog.setLoading(false)

                   val tempList = it.data?.data?.inspections ?: emptyList()

                   if(tempList[0].damages==null || tempList[0].damages!!.isEmpty())
                   {

                       binding.clNoData.visible()
                       binding.btnExportPdf.gone()
                   }else{
                       vatValue=tempList[0].vat.toString()
                       hashCode=tempList[0].uniqueHash.toString()
                       Log.e("Checkkkschhsh==>","${hashCode}")
                       binding.clNoData.gone()
                       binding.btnExportPdf.visible()


                   }
                   tempList[0].damages?.let { it1 ->
                       adapter.submitList(it1)
                       binding.damageCount.text=String.format("%02d", it1.size)
                   }




               }
               Status.ERROR -> {
                   progressDialog.setLoading(false)
                   ApisRespHandler.handleError(it.error, requireActivity())
               }
               Status.LOADING -> {
                   progressDialog.setLoading(true)
               }
           }
       })

       viewModel.deleteDamage.observe(requireActivity(), Observer {
           it ?: return@Observer
           when (it.status) {
               Status.SUCCESS -> {
                   progressDialog.setLoading(false)
                   if(clickDeletePos!=-1){

                       adapter.mainList.removeAt(clickDeletePos)
                       adapter.notifyDataSetChanged()

                       clickDeletePos=-1

                       if(adapter.mainList.isEmpty())
                       {
                           binding.clNoData.visible()
                           binding.btnExportPdf.gone()
                       }else{
                           binding.clNoData.gone()
                           binding.btnExportPdf.visible()
                       }

                       binding.damageCount.text=String.format("%02d", adapter.mainList.size)

                   }

               }
               Status.ERROR -> {

                   progressDialog.setLoading(false)
                   ApisRespHandler.handleError(it.error, requireActivity())
               }
               Status.LOADING -> {
                   progressDialog.setLoading(true)
               }
           }
       })
    }

    private fun listener(){

        binding.llBtn.setOnClickListener{


            val bundle = Bundle().apply {
                putString("id", inspectionId)
                putString("from", "add")
            }
            findNavController().navigate(R.id.damageDeatilsFragment, bundle)
        }
        binding.ivLeft.setOnClickListener {

            findNavController().popBackStack()
        }

        binding.btnExportPdf.setOnClickListener {

          //  setUpPdf()

            val url = DOWNLOAD_URL+hashCode
            val downloadsFolder = Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS)
            val fileName = url.substringAfterLast('/')+".pdf"
            val destinationFile = File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS), fileName)
            Log.e("cHehfehchh===>","${url}");
            val success =  downloadPdfFile(url, destinationFile)
          /*  if (success) {
                Toast.makeText(context, getString(R.string.pdf_saved), Toast.LENGTH_LONG).show()
            } else {
                Toast.makeText(context, getString(R.string.pdf_failed), Toast.LENGTH_LONG).show()
            }*/
        }

    }

    private fun setUpPdf(){


        try {

            val pdfDocument = PdfDocument()
            val currentLocale = Resources.getSystem().configuration.locales[0]

            if (currentLocale.layoutDirection == View.LAYOUT_DIRECTION_RTL) {
                val pageInfo = PdfDocument.PageInfo.Builder(595, 842, 1).create()
                val page = pdfDocument.startPage(pageInfo)
                val canvas = page.canvas

                val borderPaint = Paint().apply {
                    color = Color.BLACK
                    style = Paint.Style.STROKE
                    strokeWidth = 1f
                }

                val textPaint = Paint().apply {
                    textSize = 12f
                    color = Color.BLACK
                    style = Paint.Style.FILL
                    isAntiAlias = true
                    textAlign = Paint.Align.RIGHT // Important for Arabic
                }

                val whiteTextPaint = Paint(textPaint).apply {
                    color = Color.WHITE
                }

                val fillPaint = Paint().apply {
                    style = Paint.Style.FILL
                }

                val colWidths = listOf(120f, 100f, 120f, 60f, 100f, 40f) // Reversed for RTL
                val rowHeight = 40f
                val startX = 40f
                var startY = 60f

                val title = getString(R.string.damage_report_table)
                val headers = listOf(
                    getString(R.string.total_cost_in_riyals),
                    getString(R.string.unit),
                    getString(R.string.individual_cost),
                    getString(R.string.number),
                    getString(R.string.item),
                    getString(R.string.no_)
                )

                val initialRow = adapter.mainList.mapIndexed { index, damage ->
                    val no = (index + 1).toString()
                    val item = damage.subcategoryName ?: "-"
                    val number = damage.itemQuantity.toString()
                    val individualCost = damage.itemValue.toString()
                    val unit = damage.unit ?: "-"
                    val total = (damage.itemQuantity * damage.itemValue).toString()
                    listOf(total, unit, individualCost, number, item, no) // Reversed
                }

                val vatRate = vatValue.toDoubleOrNull() ?: 0.0
                val totalAmount = adapter.mainList.sumOf { it.itemQuantity * it.itemValue }
                val vatAmount = (totalAmount * (vatRate / 100)).toLong()
                val vatRow = listOf(vatAmount.toString(), "", "", "", getString(R.string.vat), "")
                val totalRow = listOf((totalAmount + vatAmount).toString(), "", "", "", getString(R.string.total_cost), "")

                val rows = initialRow + listOf(vatRow, totalRow)
                val centerAlignedColumns = setOf(0, 2, 3, 5)


                // Title
                fillPaint.color = "#285747".toColorInt()
                canvas.drawRect(startX, startY, startX + colWidths.sum(), startY + rowHeight, fillPaint)
                canvas.drawRect(startX, startY, startX + colWidths.sum(), startY + rowHeight, borderPaint)
                val titleX = startX + colWidths.sum() - (colWidths.sum() / 2f) // RTL title center
                canvas.drawText(title, titleX, startY + 25f, whiteTextPaint)
                startY += rowHeight


                fillPaint.color = "#8AA29A".toColorInt()
                var x = startX
                for ((i, header) in headers.withIndex()) {
                    val width = colWidths[i]
                    canvas.drawRect(x, startY, x + width, startY + rowHeight, fillPaint)
                    canvas.drawRect(x, startY, x + width, startY + rowHeight, borderPaint)
                    canvas.drawText(header, x + width - 5f, startY + 25f, textPaint)
                    x += width
                }
                startY += rowHeight

                for ((index, row) in rows.withIndex()) {
                    fillPaint.color = if (index % 2 == 0) Color.WHITE else "#DDE4E2".toColorInt()
                    x = startX

                    val tempCanvas = Canvas(createBitmap(1, 1))
                    val heights = row.mapIndexed { i, text ->
                        drawWrappedTextRTL(tempCanvas, text, 0f, 0f, colWidths[i], textPaint,index in centerAlignedColumns)
                    }
                    val actualRowHeight = heights.maxOrNull()?.coerceAtLeast(rowHeight) ?: rowHeight

                    x = startX
                    for ((i, cell) in row.withIndex()) {
                        val width = colWidths[i]
                        canvas.drawRect(x, startY, x + width, startY + actualRowHeight, fillPaint)
                        canvas.drawRect(x, startY, x + width, startY + actualRowHeight, borderPaint)
                        drawWrappedTextRTL(canvas, cell, x, startY + 20f, width, textPaint,i in centerAlignedColumns)
                        x += width
                    }

                    startY += actualRowHeight
                }

                pdfDocument.finishPage(page)

            } else {


                val pageInfo = PdfDocument.PageInfo.Builder(595, 842, 1).create()
                val page = pdfDocument.startPage(pageInfo)
                val canvas = page.canvas

                val borderPaint = Paint().apply {
                    color = Color.BLACK
                    style = Paint.Style.STROKE
                    strokeWidth = 1f
                }

                val textPaint = Paint().apply {
                    textSize = 12f
                    color = Color.BLACK
                    style = Paint.Style.FILL
                    isAntiAlias = true
                }

                val whiteTextPaint = Paint(textPaint).apply { color = Color.WHITE }

                val fillPaint = Paint().apply {
                    style = Paint.Style.FILL
                }

                val colWidths = listOf(40f, 100f, 60f, 120f, 100f, 120f)
                val rowHeight = 40f
                val startX = 40f
                var startY = 60f

                val title = getString(R.string.damage_report_table)
                val headers = listOf(getString(R.string.no_), getString(R.string.item), getString(R.string.number), getString(R.string.individual_cost), getString(R.string.unit), getString(R.string.total_cost_in_riyals))


                val initialRow = adapter.mainList.mapIndexed { index, damage ->
                    val no = (index + 1).toString()
                    val item = damage.subcategoryName ?: "-"
                    val number = damage.itemQuantity.toString()
                    val individualCost = damage.itemValue.toString()
                    val unit = damage.unit ?: "-"
                    val total = (damage.itemQuantity * damage.itemValue).toString()
                    listOf(no, item, number, individualCost, unit, total)
                }

                val vatRate = vatValue.toDoubleOrNull() ?: 0.0
                val totalAmount = adapter.mainList.sumOf { it.itemQuantity * it.itemValue }
                val vatAmount = (totalAmount * (vatRate / 100)).toLong()
                val vatRow = listOf("", getString(R.string.vat), "", "", "", vatAmount.toString())
                val totalRow = listOf("", getString(R.string.total_cost), "", "", "", (totalAmount + vatAmount).toString())

                val rows = initialRow + listOf(vatRow, totalRow)

                val centerAlignedColumns = setOf(0, 2, 3, 5)


                fillPaint.color = "#285747".toColorInt() // dark green
                canvas.drawRect(startX, startY, startX + colWidths.sum(), startY + rowHeight, fillPaint)
                canvas.drawRect(startX, startY, startX + colWidths.sum(), startY + rowHeight, borderPaint)
                val titleX = startX + (colWidths.sum() / 2f) - (whiteTextPaint.measureText(title) / 2f)
                canvas.drawText(title, titleX, startY + 25f, whiteTextPaint)
                startY += rowHeight


                fillPaint.color = "#8AA29A".toColorInt() // light green
                var x = startX
                for ((i, header) in headers.withIndex()) {
                    val width = colWidths[i]
                    canvas.drawRect(x, startY, x + width, startY + rowHeight, fillPaint)
                    canvas.drawRect(x, startY, x + width, startY + rowHeight, borderPaint)
                    canvas.drawText(header, x + 5f, startY + 25f, textPaint)
                    x += width
                }
                startY += rowHeight

                for ((index, row) in rows.withIndex()) {
                    fillPaint.color = if (index % 2 == 0) Color.WHITE else "#DDE4E2".toColorInt()
                    x = startX

                    val tempCanvas = Canvas(createBitmap(1, 1))
                    val heights = row.mapIndexed { i, text ->
                        drawWrappedText(tempCanvas, text, 0f, 0f, colWidths[i], textPaint,index in centerAlignedColumns)
                    }
                    val actualRowHeight = heights.maxOrNull()?.coerceAtLeast(rowHeight) ?: rowHeight

                    x = startX
                    for ((i, cell) in row.withIndex()) {
                        val width = colWidths[i]
                        canvas.drawRect(x, startY, x + width, startY + actualRowHeight, fillPaint)
                        canvas.drawRect(x, startY, x + width, startY + actualRowHeight, borderPaint)
                        drawWrappedText(canvas, cell, x, startY + 20f, width, textPaint,i in centerAlignedColumns)
                        x += width
                    }

                    startY += actualRowHeight
                }

                pdfDocument.finishPage(page)
            }


            val fileName = "damage_report_" + SimpleDateFormat("yyyyMMdd_HHmmss", Locale.getDefault()).format(
                Date()
            )
            savePdfToDocuments(pdfDocument, fileName)
        }
        catch (e:Exception)
        {
            e.printStackTrace()
        }


    }

    fun drawWrappedTextRTL(
        canvas: Canvas,
        text: String,
        x: Float,
        y: Float,
        width: Float,
        paint: Paint,
        center: Boolean = false
    ): Float {
        val words = text.split(" ")
        val lineHeight = paint.textSize + 4f
        var currentLine = ""
        var currentY = y
        val maxWidth = width - 10f
        val lines = mutableListOf<String>()

        for (word in words) {
            val testLine = if (currentLine.isEmpty()) word else "$currentLine $word"
            if (paint.measureText(testLine) > maxWidth) {
                lines.add(currentLine)
                currentLine = word
            } else {
                currentLine = testLine
            }
        }
        if (currentLine.isNotEmpty()) lines.add(currentLine)

        for (line in lines) {
            val drawX = if (center) x + (width / 2f) + (paint.measureText(line) / 2f) else x + width - 5f
            canvas.drawText(line, drawX, currentY, paint)
            currentY += lineHeight
        }

        return currentY - y
    }
    fun drawWrappedText(
        canvas: Canvas,
        text: String,
        x: Float,
        y: Float,
        width: Float,
        paint: Paint,
        center: Boolean = false
    ): Float {
        val words = text.split(" ")
        val lineHeight = paint.textSize + 4f
        var currentLine = ""
        var currentY = y
        val maxWidth = width - 10f
        val lines = mutableListOf<String>()

        for (word in words) {
            val testLine = if (currentLine.isEmpty()) word else "$currentLine $word"
            if (paint.measureText(testLine) > maxWidth) {
                lines.add(currentLine)
                currentLine = word
            } else {
                currentLine = testLine
            }
        }
        if (currentLine.isNotEmpty()) {
            lines.add(currentLine)
        }

        for (line in lines) {
            val drawX = if (center) {
                x + (width / 2f) - (paint.measureText(line) / 2f)
            } else {
                x + 5f
            }
            canvas.drawText(line, drawX, currentY, paint)
            currentY += lineHeight
        }

        return currentY - y
    }
    fun savePdfToDocuments( pdfDocument: PdfDocument, fileName: String) {
        val contentValues = ContentValues().apply {
            put(MediaStore.MediaColumns.DISPLAY_NAME, "$fileName.pdf")
            put(MediaStore.MediaColumns.MIME_TYPE, "application/pdf")
            put(MediaStore.MediaColumns.RELATIVE_PATH, Environment.DIRECTORY_DOCUMENTS)
            put(MediaStore.MediaColumns.IS_PENDING, 1)
        }

        val resolver = requireActivity().contentResolver
        val uri = resolver.insert(MediaStore.Files.getContentUri("external"), contentValues)

        uri?.let {
            resolver.openOutputStream(it)?.use { outputStream ->
                pdfDocument.writeTo(outputStream)
                pdfDocument.close()
            }

            contentValues.clear()
            contentValues.put(MediaStore.MediaColumns.IS_PENDING, 0)
            resolver.update(uri, contentValues, null, null)

            Toast.makeText(context, getString(R.string.pdf_saved), Toast.LENGTH_LONG).show()
        } ?: run {
            Toast.makeText(context, getString(R.string.pdf_failed), Toast.LENGTH_SHORT).show()
        }
    }



    fun clickItem(pos: Int) {
        val gson = Gson()
        val jsonString = gson.toJson(adapter.mainList[pos])
        val bundle = Bundle().apply {
            putString("damageData", jsonString)
            putString("id", inspectionId)
            putString("from", "edit")
        }
        findNavController().navigate(R.id.damageDeatilsFragment, bundle)

    }
    fun clickDeleteItem(pos: Int, id: String) {

        if (pos < 0 ) {
            return
        }
        clickDeletePos=pos


        requireContext().showConfirmDialog(
            title = getString(R.string.are_you_sure_you_want_to_delete)
        ) {
            viewModel.deleteInspection(id)
        }


    }



    fun downloadPdfFile(fileUrl: String, outputFile: File) {

        val okHttpClient = OkHttpClient.Builder()
            .followRedirects(true)
            .followSslRedirects(true)
            .build()

        val retrofit = Retrofit.Builder()
            .baseUrl("https://example.com/") // Placeholder, overridden by @Url
            .client(okHttpClient)
            .addConverterFactory(GsonConverterFactory.create())
            .build()

        val service = retrofit.create(FileDownloadService::class.java)
        val call = service.downloadFile(fileUrl)

        call.enqueue(object : Callback<ResponseBody> {
            override fun onResponse(call: Call<ResponseBody>, response: Response<ResponseBody>) {
                if (!response.isSuccessful) {
                    println("❌ Server returned error: ${response.code()}")
                    return
                }

                val contentType = response.body()?.contentType()?.toString()
                val contentLength = response.body()?.contentLength() ?: 0L

                println("✅ Content-Type: $contentType")
                println("✅ Content-Length: $contentLength bytes")

                if (contentType?.contains("pdf", ignoreCase = true) == true) {
                    response.body()?.let { body ->
                        val success = saveFile(body, outputFile)
                        if (success) {
                            println("✅ PDF saved at: ${outputFile.absolutePath}")
                        } else {
                            println("❌ Failed to save PDF.")
                        }
                    }
                } else {
                    println("❌ The file is not a PDF (Content-Type: $contentType).")
                }
            }

            override fun onFailure(call: Call<ResponseBody>, t: Throwable) {
                println("❌ Download error: ${t.localizedMessage}")
            }
        })
    }

    fun saveFile(body: ResponseBody, outputFile: File): Boolean {
        return try {
            val inputStream = body.byteStream()
            val outputStream = FileOutputStream(outputFile)
            val buffer = ByteArray(4096)
            var bytesRead: Int

            while (inputStream.read(buffer).also { bytesRead = it } != -1) {
                outputStream.write(buffer, 0, bytesRead)
            }

            outputStream.flush()
            outputStream.close()
            inputStream.close()

            // Verify PDF file header
            val header = FileInputStream(outputFile).use { it.readNBytes(4) }
            if (String(header) != "%PDF") {
                outputFile.delete()
                println("❌ Invalid PDF signature in downloaded file.")
                false
            } else {
                true
            }
        } catch (e: IOException) {
            e.printStackTrace()
            false
        }
    }


    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}

interface FileDownloadService {
    @Streaming
    @GET
    fun downloadFile(@Url fileUrl: String): Call<ResponseBody>
}

