package com.example.fieldagent.ui.homescreen

import android.annotation.SuppressLint
import android.content.Intent
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.os.Bundle
import android.text.Editable
import android.text.TextWatcher
import android.util.Log
import androidx.fragment.app.Fragment
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.PopupMenu
import android.widget.PopupWindow
import android.widget.TextView
import android.widget.Toast
import androidx.activity.viewModels
import androidx.core.content.ContextCompat
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.LinearLayoutManager
import com.example.fieldagent.R
import com.example.fieldagent.databinding.FragmentHomeBinding
import com.example.fieldagent.ui.homescreen.adapter.HomeAdapter
import androidx.core.graphics.drawable.toDrawable
import androidx.core.view.isVisible
import androidx.fragment.app.viewModels
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.ui.login.LoginActivity
import com.example.fieldagent.ui.login.LoginViewModel
import com.example.fieldagent.utils.IS_LOGIN
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.USER_DATA
import com.example.fieldagent.utils.dialogs.ProgressDialog
import dagger.hilt.android.AndroidEntryPoint
import java.util.HashMap
import javax.inject.Inject
import androidx.lifecycle.Observer
import androidx.lifecycle.lifecycleScope
import androidx.recyclerview.widget.RecyclerView
import com.example.fieldagent.data.models.responses.InspectionData
import com.example.fieldagent.data.models.responses.InspectionList
import com.example.fieldagent.data.models.responses.UserData
import com.example.fieldagent.databinding.PopLayoutBinding
import com.example.fieldagent.utils.AFTER
import com.example.fieldagent.utils.LIMIT_TEXT
import com.example.fieldagent.utils.PAGE_TEXT
import com.example.fieldagent.utils.PER_PAGE
import com.example.fieldagent.utils.PER_PAGE_LOAD
import com.example.fieldagent.utils.SEARCH
import com.example.fieldagent.utils.gone
import com.example.fieldagent.utils.hideShowView
import com.example.fieldagent.utils.isConnectedToInternet
import com.example.fieldagent.utils.showConfirmDialog
import com.example.fieldagent.utils.showSnackBar
import com.example.fieldagent.utils.visible
import com.google.android.material.snackbar.Snackbar
import kotlinx.coroutines.Job
import kotlinx.coroutines.delay
import kotlinx.coroutines.launch

@AndroidEntryPoint
class HomeFragment : Fragment() {

    private var _binding: FragmentHomeBinding? = null

    private val binding get() = _binding!!
    private lateinit var adapter: HomeAdapter

    @Inject
    lateinit var prefsManager: PrefsManager

    private  val viewModel: HomeViewModel by viewModels()


    private var items = ArrayList<InspectionList>()

    private var isLastPage = false

    private var isFirstPage = true

    private var isLoadingMoreItems = false
    private var pageNumber = 1

    private var searchJob: Job? = null
    private var click: String ="all"
    private var searchData: String =""
    private var selectStatus: String =""
    private var clickDeletePos: Int =0


    override fun onCreateView(
        inflater: LayoutInflater, container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {

        _binding = FragmentHomeBinding.inflate(inflater, container, false)
        return binding.root

    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        initialize()
        bindObservers()

        setAdapter()
        listener()

    }


    private  fun initialize(){

        prefsManager.save(IS_LOGIN,true)

        if (viewModel.inspections.value?.data?.data?.inspections.isNullOrEmpty()) {
            val hashMap = HashMap<String, String>()
            hashMap[LIMIT_TEXT] = PER_PAGE_LOAD
            hashMap[PAGE_TEXT] = PER_PAGE
            viewModel.getInspectionList(hashMap);

        }

        parentFragmentManager.setFragmentResultListener("refresh_home", viewLifecycleOwner) { _, _ ->
            refreshPagination()
            val hashMap = HashMap<String, String>()
            hashMap[LIMIT_TEXT] = PER_PAGE_LOAD
            hashMap[PAGE_TEXT] = PER_PAGE
            viewModel.getInspectionList(hashMap)
            binding.rvList.scrollToPosition(0)
        }


    }

    private fun setAdapter(){
        binding.rvList.layoutManager = LinearLayoutManager(requireActivity())
        adapter = HomeAdapter(this,items)
        binding.rvList.adapter = adapter
        binding.rvList.itemAnimator = null
    }

    private  fun listener(){


        binding.llBtn.setOnClickListener {
            findNavController().navigate(R.id.addSiteFragment)
        }
        binding.ivProfile.setOnClickListener {
            findNavController().navigate(R.id.profileFragment)
        }


        binding.ivFilter.setOnClickListener {


            val inflater = LayoutInflater.from(binding.ivFilter.context)
            val parent = binding.root as ViewGroup
            val popupView = PopLayoutBinding.inflate(inflater,parent,false)

            val popupWindow = PopupWindow(
                popupView.root,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT,
                true // focusable
            )


            popupView.txtCompleted.setOnClickListener {
                refreshPagination()
                click="status"
                selectStatus="true"
                callStatusSearchApi("true","")
                popupWindow.dismiss()
            }

            popupView.txtInProgress.setOnClickListener {
                refreshPagination()
                click="status"
                selectStatus="false"
                callStatusSearchApi("false","")
                popupWindow.dismiss()
            }


            popupWindow.setBackgroundDrawable(Color.TRANSPARENT.toDrawable())
            popupWindow.isOutsideTouchable = true
            popupWindow.elevation = 40f

            popupWindow.showAsDropDown(binding.ivFilter)
        }


        binding.swipeRefresh.setOnRefreshListener {

            hitApi(true)
        }



        binding.rvList.addOnScrollListener(object : RecyclerView.OnScrollListener() {
            override fun onScrolled(recyclerView: RecyclerView, dx: Int, dy: Int) {
                super.onScrolled(recyclerView, dx, dy)
                val layoutManager = binding.rvList.layoutManager as LinearLayoutManager
                val totalItemCount = layoutManager.itemCount - 1
                val lastVisibleItemPosition = layoutManager.findLastVisibleItemPosition()
                if (!isLoadingMoreItems && !isLastPage && lastVisibleItemPosition >= totalItemCount) {
                    isLoadingMoreItems = true
                    hitApi(false)
                }
            }
        })

        binding.etSearch.addTextChangedListener(object : TextWatcher {
            override fun afterTextChanged(s: Editable?) {
                // Debounce: cancel any previous search job
                searchJob?.cancel()
                searchJob = lifecycleScope.launch {
                    delay(300) // wait for user to stop typing (300ms)
                    val query = s.toString()
                    if (query.isNotBlank()) {
                        click="search"
                        searchData=query
                        refreshPagination()
                       callStatusSearchApi("false",query)
                    }else{
                        refreshPagination()
                        click="all"
                        val hashMap = HashMap<String, String>()
                        hashMap[LIMIT_TEXT] = PER_PAGE_LOAD
                        hashMap[PAGE_TEXT] = PER_PAGE
                        viewModel.getInspectionList(hashMap);
                    }
                }
            }

            override fun beforeTextChanged(s: CharSequence?, start: Int, count: Int, after: Int) {}
            override fun onTextChanged(s: CharSequence?, start: Int, before: Int, count: Int) {}
        })

    }



    private fun refreshPagination(){
        pageNumber=1;
        isLastPage = false
        isFirstPage = true
        isLoadingMoreItems = false
    }

    private fun callStatusSearchApi( value:String,search:String){
        val hashMap = HashMap<String, String>()
        if(search.isEmpty())
        {
            hashMap["isCompleted"] = value
        }else{
            hashMap[SEARCH] = searchData
        }

        hashMap[LIMIT_TEXT] = PER_PAGE_LOAD
        hashMap[PAGE_TEXT] = PER_PAGE


        viewModel.getInspectionList(hashMap);
    }
    private fun hitApi(firstHit: Boolean) {
        if (isConnectedToInternet(requireContext(), true)) {
            if (firstHit) {
                pageNumber=1
                isFirstPage = true
                isLastPage = false
                items.clear()
            }

            val hashMap = HashMap<String, String>()
            if (!isFirstPage && items.isNotEmpty())
            {
                pageNumber++
                hashMap[LIMIT_TEXT] = PER_PAGE_LOAD
                hashMap[PAGE_TEXT] = pageNumber.toString()
                if(click=="status")
                {
                    hashMap["isCompleted"] = selectStatus
                }else if(click=="search")
                {
                    hashMap[SEARCH] = searchData
                }


            }else{
                if(click=="status")
                {
                    hashMap["isCompleted"] = selectStatus
                }else if(click=="search")
                {
                    hashMap[SEARCH] = searchData
                }

                hashMap[LIMIT_TEXT] = PER_PAGE_LOAD
                hashMap[PAGE_TEXT] = PER_PAGE
            }


            viewModel.getInspectionList(hashMap)
        } else
            binding.swipeRefresh.isRefreshing = false
    }
    private fun bindObservers() {
        viewModel.inspections.observe(viewLifecycleOwner, Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {

                    Log.e("Checkkkkkkksds==in sucess=>","${items.size}")
                    binding.clLoader.root.gone()
                    binding.swipeRefresh.isRefreshing = false
                    isLoadingMoreItems = false

                    val tempList = it.data?.data?.inspections ?: emptyList()

                    if (isFirstPage) {
                        isFirstPage = false
                        items.clear()  // Clear the list on first load to prevent duplicates
                        items.addAll(tempList)
                        adapter.notifyDataSetChanged()


                    } else {
                        val oldSize = items.size
                        // Add only new items to avoid duplication
                        for (item in tempList) {
                            if (!items.contains(item)) {
                                items.add(item)
                            }
                        }
                        adapter.notifyItemRangeInserted(oldSize, items.size)
                    }

                    isLastPage = tempList.size < PER_PAGE_LOAD.toInt()
                    binding.clNoData.hideShowView(items.isEmpty())
                }
                Status.ERROR -> {
                    Log.e("Checkkkkkkksds==in erro=>","${items.size}")
                    isLoadingMoreItems = false
                    adapter.setAllItemsLoaded(true)

                    binding.swipeRefresh.isRefreshing = false
                    binding.clLoader.root.gone()
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                    Log.e("Checkkkkkkksds==in loading=>","${items.size}")
                    if (!isLoadingMoreItems && !binding.swipeRefresh.isRefreshing  && !binding.clNoData.isVisible && items.isEmpty())
                        binding.clLoader.root.visible()
                }
            }
        })


        viewModel.delete.observe(requireActivity(), Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {
                    binding.clLoader.root.gone()
                    if(clickDeletePos!=-1){

                        if (clickDeletePos >= 0 && clickDeletePos < items.size) {
                            items.removeAt(clickDeletePos)
                            adapter.notifyItemRemoved(clickDeletePos)

                            if (clickDeletePos < items.size) {
                                adapter.notifyItemRangeChanged(clickDeletePos, items.size - clickDeletePos)
                            }
                            binding.clNoData.hideShowView(items.isEmpty())
                        }
                        clickDeletePos=-1
                    }

                }
                Status.ERROR -> {
                    binding.clLoader.root.gone()
                    ApisRespHandler.handleError(it.error, requireActivity())
                }
                Status.LOADING -> {
                  binding.clLoader.root.visible()
                }
            }
        })
    }


    fun clickItem(pos: Int) {
        val item = items[pos]

        val bundle = Bundle().apply {
            putString("id", item.id.toString())  // Put data into the bundle
            putString("name", item.name.toString())  // Put data into the bundle
        }
        findNavController().navigate(R.id.damageListFragment, bundle)

    }
    fun clickDeleteItem(pos: Int,) {

        if (pos < 0 || pos >= items.size) {
            return
        }
       clickDeletePos=pos


        requireContext().showConfirmDialog(
            title = getString(R.string.are_you_sure_you_want_to_delete)
        ) {
            viewModel.deleteInspection(items[pos].id.toString())
        }

    }


    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }




}