package com.example.fieldagent.ui.homescreen.adapter

import android.util.Log
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.fragment.app.FragmentActivity
import androidx.navigation.findNavController
import androidx.navigation.fragment.findNavController
import androidx.recyclerview.widget.RecyclerView
import com.bumptech.glide.Glide
import com.example.fieldagent.R
import com.example.fieldagent.data.apis.WebService
import com.example.fieldagent.data.models.responses.InspectionData
import com.example.fieldagent.data.models.responses.InspectionList
import com.example.fieldagent.data.models.responses.UserData
import com.example.fieldagent.data.network.Config
import com.example.fieldagent.databinding.FragmentHomeBinding
import com.example.fieldagent.databinding.HomeAdapterBinding
import com.example.fieldagent.databinding.ItemPagingLoaderBinding
import com.example.fieldagent.ui.homescreen.HomeFragment
import com.example.fieldagent.utils.DateUtils.getTimeAgo
import com.example.fieldagent.utils.IMAGE_BASE_URL
import com.example.fieldagent.utils.LoadingStatus.ITEM
import com.example.fieldagent.utils.LoadingStatus.LOADING
import com.example.fieldagent.utils.showFullScreenImage

class HomeAdapter(private val  fragment: HomeFragment,private val items: ArrayList<InspectionList>) :
    RecyclerView.Adapter<RecyclerView.ViewHolder>() {

    private var allItemsLoaded = true

    override fun onBindViewHolder(holder: RecyclerView.ViewHolder, position: Int) {
        if (holder.itemViewType != LOADING)
            (holder as ViewHolder).bind(items[position])
    }


    override fun onCreateViewHolder(parent: ViewGroup, viewType: Int): RecyclerView.ViewHolder {
        return if (viewType == ITEM) {
            ViewHolder(HomeAdapterBinding.inflate(LayoutInflater.from(parent.context), parent, false))
        } else {
            ViewHolderLoader(ItemPagingLoaderBinding.inflate(LayoutInflater.from(parent.context), parent, false))
        }
    }

    override fun getItemCount(): Int = if (allItemsLoaded) items.size else items.size + 1

    override fun getItemViewType(position: Int) = if (position >= items.size) LOADING else ITEM

    inner class ViewHolder(val binding: HomeAdapterBinding) :
        RecyclerView.ViewHolder(binding.root) {

        init {
            binding.clMain.setOnClickListener {
                fragment.clickItem(bindingAdapterPosition)
            }
            binding.ivDelete.setOnClickListener {
                fragment.clickDeleteItem(bindingAdapterPosition)
            }
        }

        fun bind(item: InspectionList) = with(binding) {
            binding.txtName.text=item.name.toString()
            binding.txtLocation.text=item.location.toString()
            binding.txtTime.text=getTimeAgo(item.createdAt.toString(), context = binding.root.context)
            binding.txtPayment.text="0"
            binding.txtStatus.text = if (item.isCompleted == true) {
                binding.root.context.getString(R.string.completed)
            } else {
                binding.root.context.getString(R.string.in_progress)
            }

            Glide.with(binding.root).load(IMAGE_BASE_URL+item.image)
                .error(R.drawable.home_item) // Set a default image if the requested image fails to load
                .fallback(R.drawable.home_item)
                .into(binding.ivImage)


            binding.ivImage.setOnClickListener {
                showFullScreenImage(IMAGE_BASE_URL+item.image,binding.root.context)
            }

        }

    }

    inner class ViewHolderLoader(val binding: ItemPagingLoaderBinding) :
        RecyclerView.ViewHolder(binding.root)

    fun setAllItemsLoaded(allLoaded: Boolean) {
        allItemsLoaded = allLoaded
    }
}
