package com.example.fieldagent.ui.login

import android.app.Activity
import android.content.Intent
import android.content.res.Resources
import android.os.Bundle
import android.util.Log
import android.util.Patterns
import android.view.Gravity
import android.view.View
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.core.text.layoutDirection
import androidx.fragment.app.viewModels
import com.consultantapp.data.network.responseUtil.Status
import com.example.fieldagent.R
import com.example.fieldagent.data.models.responses.UserData
import com.example.fieldagent.data.network.ApisRespHandler
import com.example.fieldagent.databinding.ActivityLoginBinding
import com.example.fieldagent.ui.homescreen.HomeActivity
import androidx.lifecycle.Observer
import com.example.fieldagent.utils.PrefsManager
import com.example.fieldagent.utils.TOKEN
import com.example.fieldagent.utils.USER_DATA
import com.example.fieldagent.utils.dialogs.ProgressDialog
import com.example.fieldagent.utils.isConnectedToInternet
import com.example.fieldagent.utils.showSnackBar
import com.google.gson.Gson
import dagger.hilt.android.AndroidEntryPoint
import javax.inject.Inject


@AndroidEntryPoint
class LoginActivity : AppCompatActivity() {

    private lateinit var binding: ActivityLoginBinding

    @Inject
    lateinit var prefsManager: PrefsManager
    private  val viewModel: LoginViewModel by viewModels()
    private lateinit var progressDialog: ProgressDialog
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        binding = ActivityLoginBinding.inflate(layoutInflater)
        setContentView(binding.root)
        initialise()
        listeners()
        bindObservers()

    }
    private fun initialise() {
        prefsManager.removeAll()
        progressDialog = ProgressDialog(this)


        val currentLocale = Resources.getSystem().configuration.locales[0]

        if (currentLocale.layoutDirection == View.LAYOUT_DIRECTION_RTL) {
            binding.etPassword.textDirection = View.TEXT_DIRECTION_RTL
            binding.etPassword.layoutDirection = View.LAYOUT_DIRECTION_RTL
            binding.etPassword.gravity = Gravity.END

        } else {
            binding.etPassword.textDirection = View.TEXT_DIRECTION_LTR
            binding.etPassword.layoutDirection = View.LAYOUT_DIRECTION_LTR
            binding.etPassword.gravity = Gravity.START
        }

    }

    private fun listeners(){
        binding.btnSigIn.setOnClickListener {


            when{
                binding.etEmail.text.toString().isEmpty() -> {
                    binding.etEmail.showSnackBar(getString(R.string.enter_email))
                }
                !Patterns.EMAIL_ADDRESS.matcher(binding.etEmail.text.toString()).matches() -> {
                    binding.etEmail.showSnackBar(getString(R.string.enter_correct_email))
                }
                binding.etPassword.text.toString().length < 6 -> {
                    binding.etPassword.showSnackBar(getString(R.string.enter_password))
                }
                isConnectedToInternet(this, true) -> {
                    val hashMap = HashMap<String, Any>()
                    hashMap["email"] = binding.etEmail.text.toString()
                    hashMap["password"] = binding.etPassword.text.toString()

                    viewModel.login(hashMap)
                }
            }


        }
    }

    private fun bindObservers() {
        viewModel.login.observe(this, Observer {
            it ?: return@Observer
            when (it.status) {
                Status.SUCCESS -> {
                    progressDialog.setLoading(false)

                    val gson = Gson()
                    val jsonString = gson.toJson(it.data?.data)
                    prefsManager.saveString(USER_DATA,jsonString)

                    prefsManager.saveString(TOKEN,it.data?.data?.tokens?.accessToken.toString())




                    val intent = Intent(this, HomeActivity::class.java)
                    startActivity(intent)
                    finish()


                }
                Status.ERROR -> {
                    progressDialog.setLoading(false)
                    ApisRespHandler.handleError(it.error,this)
                }
                Status.LOADING -> {
                    progressDialog.setLoading(true)
                }
            }
        })
    }
}