package com.example.fieldagent.ui.login

import android.content.Intent
import android.util.Log
import androidx.core.content.ContextCompat.startActivity
import androidx.lifecycle.ViewModel
import com.example.fieldagent.data.apis.WebService
import com.example.fieldagent.data.models.responses.UserData
import com.example.fieldagent.data.network.responseUtil.ApiResponse
import com.example.fieldagent.data.network.responseUtil.ApiUtils
import com.example.fieldagent.data.network.responseUtil.Resource
import com.example.fieldagent.di.SingleLiveEvent
import com.example.fieldagent.ui.homescreen.HomeActivity
import dagger.hilt.android.lifecycle.HiltViewModel
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import javax.inject.Inject

@HiltViewModel
class LoginViewModel @Inject constructor(private val webService: WebService) : ViewModel() {

    val login by lazy { SingleLiveEvent<Resource<UserData>>() }

    fun login(hashMap: HashMap<String, Any>) {
        login.value = Resource.loading()

        webService.login(hashMap)
            .enqueue(object : Callback<UserData> {

                override fun onResponse(call: Call<UserData>,
                                        response: Response<UserData>
                ) {
                    if (response.isSuccessful) {
                        login.value = Resource.success(response.body())

                    } else {
                        login.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<UserData>, throwable: Throwable) {
                    login.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }

}