package com.example.fieldagent.ui.profile

import androidx.lifecycle.ViewModel
import com.example.fieldagent.data.apis.WebService
import com.example.fieldagent.data.models.responses.InspectionData
import com.example.fieldagent.data.models.responses.SimpleResponseData
import com.example.fieldagent.data.network.responseUtil.ApiUtils
import com.example.fieldagent.data.network.responseUtil.Resource
import com.example.fieldagent.di.SingleLiveEvent
import dagger.hilt.android.lifecycle.HiltViewModel
import retrofit2.Call
import retrofit2.Callback
import retrofit2.Response
import java.util.HashMap
import javax.inject.Inject


@HiltViewModel
class ProfileFragmentVM  @Inject constructor(private val webService: WebService) : ViewModel() {

    val profile by lazy { SingleLiveEvent<Resource<SimpleResponseData>>() }


    fun updateProfile(id:String,hashMap: HashMap<String, String>) {
        profile.value = Resource.loading()

        webService.updateProfile(id,hashMap)
            .enqueue(object : Callback<SimpleResponseData> {

                override fun onResponse(call: Call<SimpleResponseData>,
                                        response: Response<SimpleResponseData>
                ) {
                    if (response.isSuccessful) {
                        profile.value = Resource.success(response.body())

                    } else {
                        profile.value = Resource.error(
                            ApiUtils.getError(response.code(),
                                response.errorBody()?.string()))
                    }
                }

                override fun onFailure(call: Call<SimpleResponseData>, throwable: Throwable) {
                    profile.value = Resource.error(ApiUtils.failure(throwable))
                }

            })
    }




}