package com.example.fieldagent.utils

import android.app.AlertDialog
import android.app.Dialog
import android.content.Context
import android.graphics.Color
import android.graphics.drawable.ColorDrawable
import android.net.Uri
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import android.view.Window
import android.view.inputmethod.InputMethodManager
import android.widget.TextView
import android.widget.Toast
import androidx.core.content.ContextCompat
import androidx.fragment.app.Fragment
import com.example.fieldagent.R
import com.example.fieldagent.databinding.ConfirmDeleteDialogBinding
import com.google.android.material.snackbar.Snackbar
import okhttp3.MediaType.Companion.toMediaTypeOrNull
import okhttp3.RequestBody
import okhttp3.RequestBody.Companion.toRequestBody
import java.io.File
import androidx.core.net.toUri
import com.bumptech.glide.Glide
import com.example.fieldagent.databinding.DialogFullScreenImageBinding
import androidx.core.graphics.drawable.toDrawable


fun View.gone() {
    visibility = View.GONE
}

fun View.visible() {
    visibility = View.VISIBLE
}

fun View.invisible() {
    visibility = View.INVISIBLE
}

fun View.hideShowView(listIsEmpty: Boolean) {
    visibility = if (listIsEmpty) View.VISIBLE
    else View.GONE
}

fun getRequestBody(value: Any?): RequestBody {
    return (value?.toString() ?: "")
        .toRequestBody("text/plain".toMediaTypeOrNull())
}

fun View.showSnackBar(msg: String) {
    try {
        val snackBar = Snackbar.make(this, msg, Snackbar.LENGTH_LONG)
        val snackBarView = snackBar.view
        val textView =
            snackBarView.findViewById<View>(com.google.android.material.R.id.snackbar_text) as TextView
        textView.maxLines = 3
        snackBar.setAction(R.string.ok) { snackBar.dismiss() }
        snackBar.setBackgroundTint(ContextCompat.getColor(context, R.color.main_color))
        snackBar.setActionTextColor(ContextCompat.getColor(context, R.color.white))
        snackBar.show()
    } catch (e: Exception) {
        e.printStackTrace()
    }

}
fun Context.longToast(text: CharSequence) {
    Toast.makeText(this, text, Toast.LENGTH_LONG).show()
}

fun uriToFile(uri: Uri, context: Context): File? {
    val inputStream = context.contentResolver.openInputStream(uri) ?: return null
    val fileName = "temp_image_${System.currentTimeMillis()}.jpg"
    val tempFile = File(context.cacheDir, fileName)
    tempFile.outputStream().use { fileOut ->
        inputStream.copyTo(fileOut)
    }
    return tempFile
}

fun stringUriToFile(uri: String, context: Context): File? {
    val uri = uri.toUri()
    val inputStream = context.contentResolver.openInputStream(uri) ?: return null
    val fileName = "temp_image_${System.currentTimeMillis()}.jpg"
    val tempFile = File(context.cacheDir, fileName)
    tempFile.outputStream().use { fileOut ->
        inputStream.copyTo(fileOut)
    }
    return tempFile
}


fun Context.showConfirmDialog(
    title: String,
    onConfirm: () -> Unit
) {
    val dialog = Dialog(this)
    val binding = ConfirmDeleteDialogBinding.inflate(LayoutInflater.from(this))

    dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
    dialog.setContentView(binding.root)
    dialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))

    binding.tvTitle.text = title

    binding.btnYes.setOnClickListener {
        onConfirm()
        dialog.dismiss()
    }

    binding.btnNo.setOnClickListener {
        dialog.dismiss()
    }

    dialog.show()
}


fun Context.showUpdateDialog(
    title: String,
    onConfirm: () -> Unit
) {
    val dialog = Dialog(this)
    dialog.setCancelable(false)
    dialog.setCanceledOnTouchOutside(false)
    val binding = ConfirmDeleteDialogBinding.inflate(LayoutInflater.from(this))

    binding.btnNo.gone()
    binding.btnYes.setText(getString(R.string.ok))
    dialog.requestWindowFeature(Window.FEATURE_NO_TITLE)
    dialog.setContentView(binding.root)
    dialog.window?.setBackgroundDrawable(ColorDrawable(Color.TRANSPARENT))

    binding.tvTitle.text = title

    binding.btnYes.setOnClickListener {
        onConfirm()
        dialog.dismiss()
    }


    dialog.show()
}

fun showFullScreenImage(imagePath: String, context: Context) {

    val dialogBinding = DialogFullScreenImageBinding.inflate(LayoutInflater.from(context))

    val dialog = AlertDialog.Builder(context, R.style.TransparentDialog)
        .setView(dialogBinding.root)
        .create()

    val isUrl = imagePath.startsWith("http://") || imagePath.startsWith("https://")

    if (isUrl) {
        Glide.with(context)
            .load(imagePath)
            .into(dialogBinding.fullscreenImage)
    } else {
        Glide.with(context)
            .load(imagePath.toUri())
            .into(dialogBinding.fullscreenImage)
    }
    dialogBinding.closeButton.setOnClickListener { dialog.dismiss() }

    dialog.show()


    dialog.window?.setLayout(
        ViewGroup.LayoutParams.MATCH_PARENT,
        ViewGroup.LayoutParams.MATCH_PARENT
    )
    dialog.window?.setBackgroundDrawable(Color.TRANSPARENT.toDrawable())
}



fun hideKeyboard(fragment: Fragment) {
    val inputMethodManager = fragment.requireActivity().getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
    val currentFocusView = fragment.activity?.currentFocus

    // If there is a focused view, hide the keyboard
    currentFocusView?.let {
        inputMethodManager.hideSoftInputFromWindow(it.windowToken, 0)
    }
}