package com.example.fieldagent.utils

import android.content.SharedPreferences
import android.util.Log
import androidx.annotation.StringDef
import com.google.gson.Gson
import java.util.concurrent.atomic.AtomicBoolean
import javax.inject.Inject
import javax.inject.Singleton

@Singleton
class PrefsManager @Inject constructor(private val preferences: SharedPreferences, private val gson: Gson) {
    private val sharedPrefName = "ArabLife"
    companion object {
        annotation class PrefKey
        private lateinit var instance: PrefsManager
        fun get(): PrefsManager = instance }

    fun saveString(@PrefKey key: String, value: String) {
        preferences.edit().putString(key, value).apply()
    }

    fun save(@PrefKey key: String, value: Int) {
        preferences.edit().putInt(key, value).apply()
    }

    fun save(@PrefKey key: String, value: Boolean) {
        preferences.edit().putBoolean(key, value).apply()
    }

    fun save(@PrefKey key: String, `object`: Any?) {
        if (`object` == null) {
             Log.e("PrefsSave", "Attempted to save null object for key: $key")
            throw IllegalArgumentException("object is null")
        }


        save(key, gson.toJson(`object`))
    }


    fun <T> getObject(@PrefKey key: String, objectClass: Class<T>): T? {
        val jsonString = preferences.getString(key, null)
        return if (jsonString == null) {
            null
        } else {
            try {
                gson.fromJson(jsonString, objectClass)
            } catch (e: Exception) {
                throw IllegalArgumentException("Object stored with key $key is instance of other class")
            }
        }
    }
    fun getString(@PrefKey key: String, defValue: String): String =
        preferences.getString(key, defValue) ?:""

    fun getInt(@PrefKey key: String, defValue: Int): Int = preferences.getInt(key, defValue)

    fun getFloat(@PrefKey key: String, defValue: Float): Float = preferences.getFloat(key, defValue)

    fun getBoolean(@PrefKey key: String, defValue: Boolean): Boolean =
        preferences.getBoolean(key, defValue)



    fun removeAll() {
        preferences.edit().clear().apply()
    }

    fun remove(key: String) {
        preferences.edit().remove(key).apply()
    }

}